@@ -15,6 +15,13 @@ def new_archive_entry():
         ffi.entry_free(entry_p)
 
 
+def format_time(seconds, nanos):
+    """ return float of seconds.nanos when nanos set, or seconds when not """
+    if nanos:
+        return float(seconds) + float(nanos) / 1000000000.0
+    return int(seconds)
+
+
 class ArchiveEntry(object):
 
     def __init__(self, archive_p, entry_p):
@@ -28,6 +35,14 @@ class ArchiveEntry(object):
     def filetype(self):
         return ffi.entry_filetype(self._entry_p)
 
+    @property
+    def uid(self):
+        return ffi.entry_uid(self._entry_p)
+
+    @property
+    def gid(self):
+        return ffi.entry_gid(self._entry_p)
+
     def get_blocks(self, block_size=ffi.page_size):
         archive_p = self._archive_p
         buf = create_string_buffer(block_size)
@@ -91,15 +106,43 @@ class ArchiveEntry(object):
 
     @property
     def atime(self):
-        return ffi.entry_atime(self._entry_p)
+        sec_val = ffi.entry_atime(self._entry_p)
+        nsec_val = ffi.entry_atime_nsec(self._entry_p)
+        return format_time(sec_val, nsec_val)
+
+    def set_atime(self, timestamp_sec, timestamp_nsec):
+        return ffi.entry_set_atime(self._entry_p,
+                                   timestamp_sec, timestamp_nsec)
 
     @property
     def mtime(self):
-        return ffi.entry_mtime(self._entry_p)
+        sec_val = ffi.entry_mtime(self._entry_p)
+        nsec_val = ffi.entry_mtime_nsec(self._entry_p)
+        return format_time(sec_val, nsec_val)
+
+    def set_mtime(self, timestamp_sec, timestamp_nsec):
+        return ffi.entry_set_mtime(self._entry_p,
+                                   timestamp_sec, timestamp_nsec)
 
     @property
     def ctime(self):
-        return ffi.entry_ctime(self._entry_p)
+        sec_val = ffi.entry_ctime(self._entry_p)
+        nsec_val = ffi.entry_ctime_nsec(self._entry_p)
+        return format_time(sec_val, nsec_val)
+
+    def set_ctime(self, timestamp_sec, timestamp_nsec):
+        return ffi.entry_set_ctime(self._entry_p,
+                                   timestamp_sec, timestamp_nsec)
+
+    @property
+    def birthtime(self):
+        sec_val = ffi.entry_birthtime(self._entry_p)
+        nsec_val = ffi.entry_birthtime_nsec(self._entry_p)
+        return format_time(sec_val, nsec_val)
+
+    def set_birthtime(self, timestamp_sec, timestamp_nsec):
+        return ffi.entry_set_birthtime(self._entry_p,
+                                       timestamp_sec, timestamp_nsec)
 
     def _getpathname(self):
         return (ffi.entry_pathname_w(self._entry_p) or
